# loading necessary packages

library(investr)
library(data.table)
library(xlsx)

# loading dataset
mortality<-read.table(paste(Raw_data, "mortality_2.csv", sep="/"), sep=",", header = T)

# setting column qualities
mortality$Time<-as.numeric(mortality$Time)
mortality$Replica<-as.factor(mortality$Replica)
mortality<-as.data.table(mortality)
colnames(mortality)

# adding total, fungus and other causes cumulative moratlities
mortality<-mortality[ , .(Time, Incubated_aphids, Total_dead_number, Dead_from_fungus, Dead_other_cause, 
                          cummort_T = cumsum(Total_dead_number/Incubated_aphids), 
                          cummort_F = cumsum(Dead_from_fungus/Incubated_aphids), 
                          cummort_O = cumsum(Dead_other_cause/Incubated_aphids), 
                          ASIN= asin(sqrt(cumsum(Dead_from_fungus/Incubated_aphids)))), 
                      by = .(Fungus_sp, Aphid_sp, Replica)]

# checking mortalities for control
mean_cont_mort<-mortality[mortality$Fungus_sp=="Control", .(mean_cont_mort = mean(Total_dead_number/Incubated_aphids)), 
                          by = .(Fungus_sp, Aphid_sp, Time)]
mean_cont_mort$Fungus_sp<-NULL

# merging main dataset with control
mortality<-merge(mortality, mean_cont_mort, by=c("Aphid_sp", "Time"))


# mean maximum mortality rates caused by fungus
xx<-with(mortality, aggregate(cummort_F, list(Aphid_sp, Fungus_sp, Replica), max))
outp<-with(xx, aggregate(x, list(Group.1, Group.2), mean ))
outp<-cbind(outp, with(xx, aggregate(x, list(Group.1, Group.2), sd))$x)
colnames(outp)<-c("Aphid_sp", "Fungus_sp","Mean","SD")
outp$Aphid_sp<-gsub(" ", "_", outp$Aphid_sp)
outp$Fungus_sp<-gsub(" ", "_", outp$Fungus_sp)
outp


# mean maximum overall mortality rates
xx<-with(mortality, aggregate(cummort_T, list(Aphid_sp, Fungus_sp, Replica), max))
outp<-with(xx, aggregate(x, list(Group.1, Group.2), mean ))
outp<-cbind(outp, with(xx, aggregate(x, list(Group.1, Group.2), sd))$x)
colnames(outp)<-c("Aphid_sp", "Fungus_sp","Mean","SD")
outp$Aphid_sp<-gsub(" ", "_", outp$Aphid_sp)
outp$Fungus_sp<-gsub(" ", "_", outp$Fungus_sp)
outp


# mean maximum mortality rates caused by other reasons
xx<-with(mortality, aggregate(cummort_O, list(Aphid_sp, Fungus_sp, Replica), max))
outp<-with(xx, aggregate(x, list(Group.1, Group.2), mean ))
outp<-cbind(outp, with(xx, aggregate(x, list(Group.1, Group.2), sd))$x)
colnames(outp)<-c("Aphid_sp", "Fungus_sp","Mean","SD")
outp$Aphid_sp<-gsub(" ", "_", outp$Aphid_sp)
outp$Fungus_sp<-gsub(" ", "_", outp$Fungus_sp)
outp

mort2<-mortality[!mortality$Fungus_sp=="Control",]
mort2$Aphid_sp<-droplevels(mort2$Aphid_sp)
mort2$Fungus_sp<-droplevels(mort2$Fungus_sp)

# repeated measures anova
op <- options(contrasts = c("contr.helmert", "contr.poly"))

mort_anova<-aov(terms(ASIN ~ Time*Fungus_sp*Aphid_sp, keep.order = TRUE), mortality)

summary(mort_anova)
coefficients(mort_anova)

TukeyHSD(mort_anova, which = "Fungus_sp")

legend_expression<-list()
model_data<-NULL
par(ask=T)

i = 1

for (n in as.character(unique(mort2$Fungus_sp)))
{
  # pdf(file = paste(n, ".pdf", sep=""), width=8, height=8)
  
  plot(mort2$Time, sin(mort2$ASIN)^2, type="n", axes=F, xlab="Time (Hours)", ylab="Mortality", 
       ylim=c(0, 1.1), xlim=c(0, 200))
  
  axis(side=1, at=c(0, 24, 48,  72, 96, 120, 144, 168, 192), labels=c(0, 24, 48,  72, 96, 120, 144, 168, 192))
  axis(side=2, at=c(0, 0.20, 0.40, 0.60, 0.80, 1.0))
  abline(h=0.50, lty="dotted")
  
  for (m in unique(as.character(mort2$Aphid_sp)))
  {
    # setting up colors, lines, symbols for both aphid species 
    sp_col<-ifelse(m=="Sitobion avenae", "blue", "red")
    sp_pch<-ifelse(m=="Sitobion avenae", 16, 17)
    sp_lty<-ifelse(m=="Sitobion avenae", "solid", "dashed")
    # sp_mort_pch<-ifelse(m=="Sitobion avenae", 2, 3)
    sp_com<-ifelse(m=="Sitobion avenae", 0.99, 0.101)
    
    
    ### Model
    m1_subset<-mort2[mort2$Fungus_sp==n & mort2$Aphid_sp== m,]   
    attach(m1_subset)   
    M1 <- nls(ASIN ~k/(1 + exp(a +b*Time) ), start=list(k=1, a = 5, b = -0.05), data = m1_subset)
    
    # some model evaluation, residual sum of square and R²
    RSS <- sum(residuals(M1)^2)
    TSS <- sum((ASIN - mean(ASIN))^2)
    R.square <- 1 - (RSS/TSS)
    
    
    cat(n, m)
    cat("\n")
    cat("Measured")
    cat("\n")
    print(summary(M1))
    fitted(M1)
    cat("\n\n")
    
    mort_mean<-aggregate(sin(ASIN)^2, list(as.factor(Time)), mean)
    mort_sd<-aggregate(sin(ASIN)^2, list(as.factor(Time)), sd)
    Time_mes<-as.numeric(paste(mort_mean$Group.1))
    
    
    points(Time_mes, mort_mean$x, pch=sp_pch, col=sp_col)
    
    
    y2<-as.data.frame(predFit(M1, newdata = data.frame(Time = seq(0,200, length.out = 100)), interval = "confidence", 
                              level = 0.95, adjust = "none"))
    
    
    
    lines(seq(0,200, length.out = 100), sin(y2$fit)^2, col=sp_col, lwd=2, lty=sp_lty) 
    lines(seq(0,200, length.out = 100), sin(y2$lwr)^2, col=sp_col, lty="dotted")
    lines(seq(0,200, length.out = 100), sin(y2$upr)^2, col=sp_col, lty="dotted")
    
    
    
    # median lethal dose (one does not reach 50%, therefor it must be tried first)
    
    LT50<-try(invest(M1, y0 = asin(sqrt(0.5)), interval = "Wald"), silent=T) 
    
    
    if (!class(LT50) == "try-error")
    {
      
      abline(v=LT50$estimate, col=sp_col, lty= sp_lty)
      points(LT50$estimate, sp_com, col=sp_col, pch=sp_pch, lwd=3)
      segments(x0 = LT50$lower, y0 = sp_com,
               x1= LT50$upper,  y1 = sp_com,
               lwd = 2, col=sp_col)
      segments(x0 = LT50$lower, y0 = sp_com-0.01,
               x1= LT50$lower,  y1 = sp_com+0.01,
               lwd = 2, col = sp_col)
      segments(x0 = LT50$upper, y0 = sp_com-0.01,
               x1= LT50$upper,  y1 = sp_com+0.01,
               lwd = 2, col = sp_col)
      text(LT50$estimate, sp_com, 
           paste("LT50=", round(LT50$estimate,2), "hours\n (CL= ", round(LT50$lower,3),", ", round(LT50$upper,3), ")", sep=""), 
           col=sp_col, cex=0.6, pos=3)
    }  else
    {LT50$estimate<-NA}
    
    
    M1$fit<-y2$fit
    M1$spF<-n
    M1$spA<-m
    assign(paste("MOD", i, sep="_"), M1)
    
    
    detach(m1_subset)
  
    
    sp_comb<-paste(
      paste(sapply(strsplit(m, " "), function(x) substr(x, 1,1) ), collapse = "") ,
      paste(sapply(strsplit(n, " "), function(x) substr(x, 1,1) ), collapse = ""), sep="-")
    
    legend_expr<-bquote("M"[.(sp_comb)]*"="~.
                        (round(coef(M1)[1],3)) ~ 
                          "/(1+e"^
                          .(paste("(", round(coef(M1)[2],3),round(coef(M1)[3],3), ")", sep=""))*"),"
                        ~"R"^"2"*~"="~.(round(R.square, 2))
    )
    
    
    
    
    legend_expression[m]=as.expression(legend_expr)
    
    data_line<-NULL
    
    data_line<-data.frame(Aphid_sp=m, 
                          Fungus_sp=n,
                          Fungus_abbr = substr(n, 1,4),
                          Aphid_abbr = substr(m, 1,4),
                          Model_R_sq = R.square,
                          K=round(coef(M1)[1],3), 
                          K_SD = round(coef(summary(M1))[4],3),
                          K_t_val = round(coef(summary(M1))[7],3),
                          K_p_val = round(coef(summary(M1))[10],3),
                          a=round(coef(M1)[2],3), 
                          a_SD = round(coef(summary(M1))[5],3),
                          a_t_val = round(coef(summary(M1))[8],3),
                          a_p_val = round(coef(summary(M1))[11],3),
                          b=round(coef(M1)[3],3), 
                          b_SD = round(coef(summary(M1))[6],3),
                          b_t_val = round(coef(summary(M1))[9],3),
                          b_p_val = round(coef(summary(M1))[12],3),
                          LT50 = ifelse(!is.na(LT50$estimate), round(LT50$estimate,2), NA), 
                          LT50_lower = ifelse(!is.na(LT50$estimate), round(LT50$lower,3), NA),
                          LT50_upper = ifelse(!is.na(LT50$estimate), round(LT50$upper,3), NA), 
                          LT50_se = ifelse(!is.na(LT50$estimate), round(LT50$se,3), NA)
    )
    
    rownames(data_line)<-NULL                      
    
    model_data<-rbind(model_data, data_line)
    i<-i+1 
    
  }
  legend("topleft", c(as.expression(legend_expression)), col=c("red", "blue"), 
         lty=c("dashed", "solid"), lwd=2, seg.len=2, cex=0.9, bg="white")
  
  # dev.off()   
  
}


#### Comparing predictid values of the generated models ####
df1<-data.frame(fit=MOD_1$fit, model="MOD1", species=paste(substr(MOD_1$spF, 1,4), substr(MOD_1$spA, 1,4), sep="_"))
df2<-data.frame(fit=MOD_2$fit, model="MOD2", species=paste(substr(MOD_2$spF, 1,4), substr(MOD_2$spA, 1,4), sep="_"))
df3<-data.frame(fit=MOD_3$fit, model="MOD3", species=paste(substr(MOD_3$spF, 1,4), substr(MOD_3$spA, 1,4), sep="_"))
df4<-data.frame(fit=MOD_4$fit, model="MOD4", species=paste(substr(MOD_4$spF, 1,4), substr(MOD_4$spA, 1,4), sep="_"))

fitted_values_mod<-rbind(df1, df2, df3, df4)
nls_AOV<-aov(fit~species, data=fitted_values_mod)

summary(nls_AOV)
TukeyHSD(nls_AOV)

#### Model comparison ends ####

#### Comparing LT50s ####
reverse_t_test<-function(m1, m2, se1, se2, n1, n2)
{
  df <- (n1+n2)-2
  t <- (m1-m2)/sqrt(se1^2+se2^2)
  
  return(pt(t, df))
  
}

get_test<- function(x, y) # x is the pair of factors to compare, y is data frame we are getting the data from
{
  comp1_mean<-y[paste(y$Fungus_abbr, y$Aphid_abbr)==x[1], "LT50"]    
  comp2_mean<-y[paste(y$Fungus_abbr, y$Aphid_abbr)==x[2], "LT50"]
  comp1_se<-y[paste(y$Fungus_abbr, y$Aphid_abbr)==x[1], "LT50_se"]   
  comp2_se<- y[paste(y$Fungus_abbr, y$Aphid_abbr)==x[2], "LT50_se"] 
  
  ttest<-reverse_t_test(comp1_mean, comp2_mean, comp1_se, comp2_se, 5, 5)
  ttest
}




combi<-as.data.frame(combn(unique(paste(model_data$Fungus_abbr, model_data$Aphid_abbr)), 2))
colnames(combi)<-apply(combi, 2, function(x) paste(gsub(" ", "", x[1]), gsub(" ", "", x[2]), sep="_"))


dataf<-apply(combi, 2, function(x) get_test(x, model_data))


#### LT50 comparison ends ####

#### Start of ANOVA max ####
Dead_max<-with(mort2, 
               aggregate(ASIN, list(Fungus_sp=Fungus_sp, Aphid_sp=Aphid_sp, Replica=Replica), max)
)
colnames(Dead_max)[4]<-"Max_deaths"

levels(Dead_max$Fungus_sp)<-c("Entap", "Pandna")
levels(Dead_max$Aphid_sp)<-c("Rhopp", "Sita")

ANOVA1<-aov(Max_deaths ~ Fungus_sp*Aphid_sp, data=Dead_max)
summary(ANOVA1)

TukeyHSD(ANOVA1)
#### End of ANOVA max ####